import { Component, Input, OnInit } from '@angular/core';
import * as moment from 'moment';

declare const google;

@Component({
	selector: 'app-chart',
	templateUrl: './chart.component.html',
	styleUrls: ['./chart.component.css']
})

export class ChartComponent implements OnInit {
	@Input()stock;

	constructor() { }

	ngOnInit() {
		google.charts.load('current', {'packages': ['corechart', 'line']});

		google.charts.setOnLoadCallback(() => {
			const data = this.stock ? this.initChartData() : this.initNullChart();
			this.drawChart(data);
		});
	}

	initChartData() {
		const chartData = this.stock.chart.reduce((a, price) => {
			a.push([this.parseDate(price.date), price.close]);
			return a;
		}, []);

		return google.visualization.arrayToDataTable([
			['Date', 'Close'],
			...chartData
		]);
	}

	parseDate = (date) => moment(date).format('MMM DD');

	drawChart(data) {
		const options = {
			lineWidth: 5,
			baselineColor: 'transparent',
			focusTarget: 'category',
			hAxis: {
				showTextEvery: 5,
				gridlines: {
					color: '#333',
					count: 4
				},
				minorGridlines: {
					color: '#FFFFFF',
					count: 4
				}
			},
			legend: 'none',
			chartArea: {
				left: 0,
				top: 0,
				width: '100%',
				height: '100%',
			},
			backgroundColor: 'transparent',
			vAxis: {
				gridlines: {
					color: 'transparent'
				}
			},
			curveType: 'function'
		};

		const chart = new google.visualization.LineChart(document.getElementById('chart_div'));

		google.visualization.events.addOneTimeListener(chart, 'ready', () => {
			this.addChartGradient(chart);
		});

		chart.draw(data, options);
	}


	addChartGradient(chart) {
		const chartDiv = chart.getContainer();
		const svg = chartDiv.getElementsByTagName('svg')[0];
		const properties = {
			id: 'chartGradient',
			x1: '0%',
			y1: '0%',
			x2: '0%',
			y2: '100%',
			stops: [
				{ offset: '5%', 'stop-color': '#FFBF8D' },
				{ offset: '95%', 'stop-color': '#CE74C6' }
			]
		};


		this.createGradient(svg, properties);
		const chartPath = svg.getElementsByTagName('path')[0];
		chartPath.setAttribute('stroke', 'url(#chartGradient)');
	}


	createGradient(svg, properties) {
		const svgNS = svg.namespaceURI;
		const grad = document.createElementNS(svgNS, 'linearGradient');
		grad.setAttribute('id', properties.id);
		['x1', 'y1', 'x2', 'y2'].forEach(function(name) {
			if (properties.hasOwnProperty(name)) {
				grad.setAttribute(name, properties[name]);
			}
		});
		for (let i = 0; i < properties.stops.length; i++) {
			const attrs = properties.stops[i];
			const stop = document.createElementNS(svgNS, 'stop');
			for (const attr in attrs) {
				if (attrs.hasOwnProperty(attr)) {
					stop.setAttribute(attr, attrs[attr]);
				}
			}
			grad.appendChild(stop);
		}

		const defs = svg.querySelector('defs') ||
			svg.insertBefore(document.createElementNS(svgNS, 'defs'), svg.firstChild);
		return defs.appendChild(grad);
	}

	initNullChart() {
		return google.visualization.arrayToDataTable([
			['Date', 'Close'],
			['0', 1],
			['1', 1],
		]);
	}
}
